@@ -9,12 +9,21 @@ import (
 	"testing"
 
 	imageregistryv1 "github.com/openshift/api/imageregistry/v1"
+	networkv1 "github.com/openshift/api/network/v1"
 	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
+	policyv1beta1 "k8s.io/api/policy/v1beta1"
 	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/apimachinery/pkg/runtime/schema"
+	"k8s.io/apimachinery/pkg/runtime/serializer/yaml"
+	dynamicfake "k8s.io/client-go/dynamic/fake"
 	"k8s.io/klog"
+	"k8s.io/apimachinery/pkg/util/intstr"
+	kubefake "k8s.io/client-go/kubernetes/fake"
 
 	imageregistryfake "github.com/openshift/client-go/imageregistry/clientset/versioned/fake"
+	networkfake "github.com/openshift/client-go/network/clientset/versioned/fake"
 	"github.com/openshift/insights-operator/pkg/record"
 	"github.com/openshift/insights-operator/pkg/utils"
 )
@@ -166,6 +175,57 @@ func TestGatherClusterPruner(t *testing.T) {
 
 }
 
+func TestGatherPodDisruptionBudgets(t *testing.T){
+	coreClient := kubefake.NewSimpleClientset()
+
+	fakeNamespace := "fake-namespace"
+
+	// name -> MinAvailabel
+	fakePDBs := map[string]string {
+		"pdb-four": "4",
+		"pdb-eight": "8",
+		"pdb-ten": "10",
+	}
+	for name, minAvailable := range fakePDBs{
+		_, err := coreClient.PolicyV1beta1().
+			PodDisruptionBudgets(fakeNamespace).
+			Create(&policyv1beta1.PodDisruptionBudget{
+				ObjectMeta: metav1.ObjectMeta{
+					Namespace: fakeNamespace,
+					Name:      name,
+				},
+				Spec: policyv1beta1.PodDisruptionBudgetSpec{
+					MinAvailable: &intstr.IntOrString{StrVal: minAvailable},
+				},
+			})
+		if err != nil {
+			t.Fatalf("unable to create fake pdbs: %v", err)
+		}
+	}
+
+	gatherer := &Gatherer{policyClient: coreClient.PolicyV1beta1()}
+
+	records, errs := GatherPodDisruptionBudgets(gatherer)()
+	if len(errs) > 0 {
+		t.Errorf("unexpected errors: %#v", errs)
+		return
+	}
+	if len(records) != len(fakePDBs) {
+		t.Fatalf("unexpected number of records gathered: %d (expected %d)", len(records), len(fakePDBs))
+	}
+	for _, rec := range records {
+		pdba, ok := rec.Item.(PodDisruptionBudgetsAnonymizer)
+		if !ok {
+			t.Fatal("pdb item has invalid type")
+		}
+		name := pdba.PodDisruptionBudget.ObjectMeta.Name
+		minAvailable := pdba.PodDisruptionBudget.Spec.MinAvailable.StrVal
+		if pdba.PodDisruptionBudget.Spec.MinAvailable.StrVal !=  fakePDBs[name] {
+			t.Fatalf("pdb item has mismatched MinAvailable value, %q != %q", fakePDBs[name], minAvailable)
+		}
+	}
+}
+
 func TestGatherClusterImageRegistry(t *testing.T) {
 	tests := []struct {
 		name       string
@@ -306,6 +366,98 @@ func TestGatherClusterImageRegistry(t *testing.T) {
 	}
 
 }
+func TestGatherMachineSet(t *testing.T) {
+	var machineSetYAML = `
+apiVersion: machine.openshift.io/v1beta1
+kind: MachineSet
+metadata:
+    name: test-worker
+`
+	gvr := schema.GroupVersionResource{Group: "machine.openshift.io", Version: "v1beta1", Resource: "machinesets"}
+	client := dynamicfake.NewSimpleDynamicClient(runtime.NewScheme())
+	decUnstructured := yaml.NewDecodingSerializer(unstructured.UnstructuredJSONScheme)
+
+	testMachineSet := &unstructured.Unstructured{}
+
+	_, _, err := decUnstructured.Decode([]byte(machineSetYAML), nil, testMachineSet)
+	if err != nil {
+		t.Fatal("unable to decode machineset ", err)
+	}
+	_, err = client.Resource(gvr).Create(testMachineSet, metav1.CreateOptions{})
+	if err != nil {
+		t.Fatal("unable to create fake machineset ", err)
+	}
+
+	gatherer := &Gatherer{dynamicClient: client}
+	records, errs := GatherMachineSet(gatherer)()
+	if len(errs) > 0 {
+		t.Errorf("unexpected errors: %#v", errs)
+		return
+	}
+	if len(records) != 1 {
+		t.Fatalf("unexpected number or records %d", len(records))
+	}
+	if records[0].Name != "machinesets/test-worker" {
+		t.Fatalf("unexpected machineset name %s", records[0].Name)
+	}
+}
+
+func TestGatherHostSubnet(t *testing.T) {
+	testHostSubnet := networkv1.HostSubnet{
+		Host:        "test.host",
+		HostIP:      "10.0.0.0",
+		Subnet:      "10.0.0.0/23",
+		EgressIPs:   []string{"10.0.0.0", "10.0.0.1"},
+		EgressCIDRs: []string{"10.0.0.0/24", "10.0.0.0/24"},
+	}
+	client := networkfake.NewSimpleClientset()
+	_, err := client.NetworkV1().HostSubnets().Create(&testHostSubnet)
+	if err != nil {
+		t.Fatal("unable to create fake hostsubnet")
+	}
+
+	gatherer := &Gatherer{networkClient: client.NetworkV1()}
+
+	records, errs := GatherHostSubnet(gatherer)()
+	if len(errs) > 0 {
+		t.Errorf("unexpected errors: %#v", errs)
+		return
+	}
+	if len(records) != 1 {
+		t.Fatalf("unexpected number or records %d", len(records))
+	}
+	item, err := records[0].Item.Marshal(context.TODO())
+	var gatheredHostSubnet networkv1.HostSubnet
+	_, _, err = networkSerializer.LegacyCodec(networkv1.SchemeGroupVersion).Decode(item, nil, &gatheredHostSubnet)
+	if err != nil {
+		t.Fatalf("failed to decode object: %v", err)
+	}
+	if gatheredHostSubnet.HostIP != "xxxxxxxx" {
+		t.Fatalf("Host IP is not anonymized %s", gatheredHostSubnet.HostIP)
+	}
+	if gatheredHostSubnet.Subnet != "xxxxxxxxxxx" {
+		t.Fatalf("Host Subnet is not anonymized %s", gatheredHostSubnet.Subnet)
+	}
+	if len(gatheredHostSubnet.EgressIPs) != len(testHostSubnet.EgressIPs) {
+		t.Fatalf("unexpected number of egress IPs gathered %s", gatheredHostSubnet.EgressIPs)
+	}
+
+	if len(gatheredHostSubnet.EgressCIDRs) != len(testHostSubnet.EgressCIDRs) {
+		t.Fatalf("unexpected number of egress CIDRs gathered %s", gatheredHostSubnet.EgressCIDRs)
+	}
+
+	for _, ip := range gatheredHostSubnet.EgressIPs {
+		if ip != "xxxxxxxx" {
+			t.Fatalf("Egress IP is not anonymized %s", ip)
+		}
+	}
+
+	for _, cidr := range gatheredHostSubnet.EgressCIDRs {
+		if cidr != "xxxxxxxxxxx" {
+			t.Fatalf("Egress CIDR is not anonymized %s", cidr)
+		}
+	}
+}
 
 func ExampleGatherMostRecentMetrics_Test() {
 	b, err := ExampleMostRecentMetrics()
